<?php
/* Exit if accessed directly */
if (!defined('ABSPATH'))
    exit;

/* Check if WooCommerce is active */
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
if ( is_plugin_active( 'woocommerce/woocommerce.php' ) ) {

    /* Init plugin */
    function woocommerce_dpdfrance_classic_init()
    {
        if (!class_exists('WC_dpdfrance_classic'))
        {
            /* Main class */
            class WC_dpdfrance_classic extends WC_Shipping_Method
            {
                function __construct() {
                    $this->id                           = 'dpdfrance_classic';
                    $this->table_rate_option            = $this->id.'_table_rates';
                    $this->last_table_rate_id_option    = $this->id.'_last_table_rate_id';
                    $this->zones_option                 = $this->id.'_zones';
                    $this->last_zone_id_option          = $this->id.'_last_zone_id';
                    $this->method_title                 = __('DPD CLASSIC (France only)', 'woocommerce-dpdfrance');
                    $this->method_description           = __('DPD CLASSIC delivery plugin for WooCommerce - France vendors only', 'woocommerce-dpdfrance' );
                    $this->get_zones();
                    $this->get_last_zone_id();
                    $this->get_table_rates();
                    $this->get_last_table_rate_id();

                    $this->init();

                    foreach ($this->settings as $key => $val)
                        $this->$key = $val;
                }

                function init()
                {
                    // Load the settings API
                    $this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
                    $this->init_settings(); // This is part of the settings API. Loads settings you previously init.

                    $this->enabled = $this->settings['enabled'];
                    $this->title = $this->settings['title']; // Shown in drop down and admin order screen

                    // Save settings in admin if you have any defined
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_zones' ) );
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_table_rates' ) );
                }

                /* Generates dropdown menus */
                function generate_options()
                {
                    $option_arr = array();

                    /* Zones */
                    foreach($this->zones as $option):
                        $option_arr['table_rate_zone'][esc_attr($option['id'])] = esc_js($option['name']);
                    endforeach;

                    $option_arr['table_rate_zone']['0'] = __( 'Everywhere Else', 'woocommerce-dpdfrance');

                    /* Countries */
                    foreach (WC()->countries->get_shipping_countries() as $id => $value) :
                        $option_arr['country'][esc_attr($id)] = esc_js($value);
                    endforeach;

                    /* Table rates */
                    $option_arr['rate_basis']['weight'] = sprintf(__( 'Weight (%s)', 'woocommerce-dpdfrance' ), get_option('woocommerce_weight_unit'));
                    $option_arr['rate_basis']['price'] = sprintf(__( 'Total (%s)', 'woocommerce-dpdfrance' ), get_woocommerce_currency_symbol());

                    return $option_arr;
                }

                /* Generates HTML code for top of configuration page */
                function admin_options() {
                ?>
                    <script type="text/javascript">

                        var options = <?php echo json_encode($this->generate_options()); ?>;

                        function generateSelectOptionsHtml(options, selected) {
                            var html;
                            var selectedHtml;

                            for (var key in options) {
                                var value = options[key];

                                if (selected instanceof Array) {
                                    if (selected.indexOf(key) != -1) {
                                        selectedHtml = ' selected="selected"';
                                    } else {
                                        selectedHtml = '';
                                    }
                                } else {
                                    if (key == selected) {
                                        selectedHtml = ' selected="selected"';
                                    } else {
                                        selectedHtml = '';
                                    }
                                }

                                html += '<option value="' + key +'"' + selectedHtml + '>' + value + '</option>';
                            }

                            return html;
                        }
                    </script>
                    <style>
                        .debug-col {
                            display: none;
                        }
                        table.shippingrows tr th {
                            padding-left: 10px;
                        }
                        .zone td {
                            vertical-align: top;
                        }
                        .zone textarea {
                            width: 100%;
                        }

                    </style>
                    <h2><?php _e('DPD CLASSIC (France only)', 'woocommerce-dpdfrance'); ?></h2>
                    <div class="updated woocommerce-message">
                        <p><strong><?php _e('Welcome to DPD.', 'woocommerce-dpdfrance'); ?></strong> <?php _e('The plugin documentation is available here :', 'woocommerce-dpdfrance'); ?><a target="_blank" href="<?php echo OS_DPDFRANCE_ROOT_URL;?>/assets/docs/readme_dpdfrance_woocommerce.pdf"><img src="<?php echo OS_DPDFRANCE_ROOT_URL;?>/assets/img/admin/pdf.png"/></a></p>
                    </div>
                    <table class="form-table">
                    <?php $this->generate_settings_html(); ?>
                    </table>

                <?php
                }

                /* Generates HTML code for zone settings table */
                function generate_zones_table_html() {
                    ob_start();
                    ?>
                    <tr valign="top">
                        <th scope="row" class="titledesc"><?php _e( 'Shipping Zones', 'woocommerce-dpdfrance' ); ?></th>
                        <td class="forminp" id="<?php echo $this->id; ?>_zones">
                            <p style="padding-bottom: 10px;"><?php _e( 'After adding a shipping zone, hit "Save changes" so that it appears as an option in the table rate section.', 'woocommerce-dpdfrance' ); ?></p>
                            <table class="shippingrows widefat" cellspacing="0">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:100%;">
                                <thead>
                                    <tr>
                                        <th class="check-column"><input type="checkbox"></th>
                                        <!--<th class="debug-col"><?php _e( 'ID', 'woocommerce-dpdfrance' ); ?></th>-->
                                        <th><?php _e( 'Name', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Shipping zone name, will appear in table rates table.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Countries', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Add one or more countries that are part of this shipping zone.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                    </tr>
                                </thead>
                                <tfoot>
                                    <tr>
                                        <th colspan="3"><a href="#" class="add button"><?php _e( 'Add Shipping Zone', 'woocommerce-dpdfrance' ); ?></a> <a href="#" class="remove button"><?php _e( 'Delete Selected Zones', 'woocommerce-dpdfrance' ); ?></a></th>
                                    </tr>
                                </tfoot>
                                <tbody class="zones">
                                    <tr class="zone">
                                        <th></th>
                                        <!--<td class="debug-col">0</td>-->
                                        <td><div style="width: 200px;"><?php _e( 'Default Zone (everywhere else)', 'woocommerce-dpdfrance' ); ?></div></td>
                                        <td><em><?php _e( 'All allowed countries', 'woocommerce-dpdfrance' ); ?></em></td>
                                    </tr>
                                </tbody>
                            </table>
                            <script type="text/javascript">

                                var lastZoneId = <?php echo $this->last_zone_id; ?>;

                                <?php
                                foreach ($this->zones as $zone):
                                    $js_array = json_encode($zone);
                                    echo "jQuery('#{$this->id}_zones table tbody tr:last').before(addZoneRowHtml(false, {$js_array}));\n";
                                endforeach;
                                ?>

                                function addZoneRowHtml(isNew, rowArr) {

                                    if (isNew) {
                                        lastZoneId++;
                                        rowArr = {};
                                        rowArr['id'] = lastZoneId;
                                        rowArr['name'] = '';
                                        rowArr['country'] = '';
                                        rowArr['type'] = 'country';
                                        rowArr['include'] = '';
                                        rowArr['exclude'] = '';
                                        rowArr['enabled'] = '1';
                                    }

                                    var size = jQuery('#<?php echo $this->id; ?>_zones tbody .zone').size();
                                    var html = '\
                                            <tr class="zone">\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_id[' + size + ']" value="' + rowArr['id'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_type[' + size + ']" value="' + rowArr['type'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_include[' + size + ']" value="' + rowArr['include'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_exclude[' + size + ']" value="' + rowArr['exclude'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_enabled[' + size + ']" value="' + rowArr['enabled'] + '" />\
                                                <th class="check-column"><input type="checkbox" name="select" /></th>\
                                                <!--<td class="debug-col">\
                                                    ' + rowArr['id'] + '\
                                                </td>-->\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_zone_name[' + size + ']" value="' + rowArr['name'] + '" size="30" placeholder="" />\
                                                </td>\
                                                <td style="overflow:visible;">\
                                                    <select multiple="multiple" name="<?php echo $this->id; ?>_zone_country[' + size + '][]" class="multiselect chosen_select">\
                                                        ' + generateSelectOptionsHtml(options['country'], rowArr['country']) + '\
                                                    </select>\
                                                </td>\
                                            </tr>';
                                    return html;
                                }

                                jQuery(function() {

                                    jQuery('#<?php echo $this->id; ?>_zones').on( 'click', 'a.add', function(){

                                        jQuery('#<?php echo $this->id; ?>_zones table tbody tr:last').before(addZoneRowHtml(true, false));

                                        if (jQuery().chosen) {
                                            jQuery("select.chosen_select").chosen({
                                                width: '350px',
                                                disable_search_threshold: 5
                                            });
                                        } else {
                                            jQuery("select.chosen_select").select2();
                                        }

                                        return false;
                                    });

                                    // Remove row
                                    jQuery('#<?php echo $this->id; ?>_zones').on( 'click', 'a.remove', function(){

                                        var answer = confirm("<?php _e( 'Delete the selected zones?', 'woocommerce-dpdfrance' ); ?>");
                                        if (answer) {
                                            jQuery('#<?php echo $this->id; ?>_zones table tbody tr th.check-column input:checked').each(function(i, el){
                                                jQuery(el).closest('tr').remove();
                                            });
                                        }
                                        return false;
                                    });

                                });
                            </script>
                        </td>
                    </tr>
                    <?php
                    return ob_get_clean();
                }

                /* Generates HTML for table_rate settings table */
                function generate_table_rates_table_html() {
                    ob_start();
                    ?>
                    <tr valign="top">
                        <th scope="row" class="titledesc"><?php _e( 'Table Rates', 'woocommerce-dpdfrance' ); ?></th>
                        <td class="forminp" id="<?php echo $this->id; ?>_table_rates">
                            <table class="shippingrows widefat" cellspacing="0">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:100%;">
                                <thead>
                                    <tr>
                                        <th class="check-column"><input type="checkbox"></th>
                                        <th class="debug-col"><?php _e( 'ID', 'woocommerce-dpdfrance' ); ?></th>
                                        <th><?php _e( 'Zone', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Shipping zone, as defined in Shipping Zones table.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Condition', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Choose which metric to base your table rate on.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Min', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Minimum, in decimal format. Inclusive.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Max', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Maximum, in decimal format. Inclusive. To impose no upper limit, use *".', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Cost', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Cost, excluding tax.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                    </tr>
                                </thead>
                                <tfoot>
                                    <tr>
                                        <th colspan="7"><a href="#" class="add button"><?php _e( 'Add Table Rate', 'woocommerce-dpdfrance' ); ?></a> <a href="#" class="remove button"><?php _e( 'Delete Selected Rates', 'woocommerce-dpdfrance' ); ?></a></th>
                                    </tr>
                                </tfoot>
                                <tbody class="table_rates">

                                </tbody>
                            </table>
                            <script type="text/javascript">

                                var lastTableRateId = <?php echo $this->last_table_rate_id; ?>;

                                <?php
                                foreach ($this->table_rates as $table_rate):
                                    $js_array = json_encode($table_rate);
                                    echo "jQuery(addTableRateRowHtml(false, {$js_array})).appendTo('#{$this->id}_table_rates table tbody');\n";
                                endforeach;
                                ?>

                                function addTableRateRowHtml(isNew, rowArr) {

                                    if (isNew) {
                                        lastTableRateId++;
                                        rowArr = {};
                                        rowArr['id'] = lastTableRateId;
                                        rowArr['zone'] = '<?php echo (!empty($this->zones[0]['id'])) ? $this->zones[0]['id'] : 0; ?>';
                                        rowArr['basis'] = 'weight';
                                        rowArr['min'] = '0';
                                        rowArr['max'] = '*';
                                        rowArr['cost'] = '0';
                                        rowArr['enabled'] = '1';
                                    }

                                    var size = jQuery('#<?php echo $this->id; ?>_table_rates tbody .table_rate').size();
                                    var html = '\
                                            <tr class="table_rate">\
                                                <input type="hidden" name="<?php echo $this->id; ?>_table_rate_id[' + size + ']" value="' + rowArr['id'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_table_rate_enabled[' + size + ']" value="' + rowArr['enabled'] + '" />\
                                                <th class="check-column"><input type="checkbox" name="select" /></th>\
                                                <td class="debug-col">\
                                                    ' + rowArr['id'] + '\
                                                </td>\
                                                <td>\
                                                    <select name="<?php echo $this->id; ?>_table_rate_zone[' + size + ']">\
                                                        ' + generateSelectOptionsHtml(options['table_rate_zone'], rowArr['zone']) + '\
                                                    </select>\
                                                </td>\
                                                <td>\
                                                    <select name="<?php echo $this->id; ?>_table_rate_basis[' + size + ']">\
                                                        ' + generateSelectOptionsHtml(options['rate_basis'], rowArr['basis']) + '\
                                                    </select>\
                                                </td>\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_table_rate_min[' + size + ']" value="' + rowArr['min'] + '" placeholder="0" size="4" />\
                                                </td>\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_table_rate_max[' + size + ']" value="' + rowArr['max'] + '" placeholder="*" size="4" />\
                                                </td>\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_table_rate_cost[' + size + ']" value="' + rowArr['cost'] + '" placeholder="<?php echo wc_format_localized_price( 0 ); ?>" size="4" class="wc_input_price" />\
                                                </td>\
                                            </tr>';
                                    return html;
                                }

                                jQuery(function() {

                                    jQuery('#<?php echo $this->id; ?>_table_rates').on( 'click', 'a.add', function(){

                                        jQuery(addTableRateRowHtml(true, false)).appendTo('#<?php echo $this->id; ?>_table_rates table tbody');

                                        return false;
                                    });

                                    // Remove row
                                    jQuery('#<?php echo $this->id; ?>_table_rates').on( 'click', 'a.remove', function(){
                                        var answer = confirm("<?php _e( 'Delete the selected rates?', 'woocommerce-dpdfrance' ); ?>");
                                        if (answer) {
                                            jQuery('#<?php echo $this->id; ?>_table_rates table tbody tr th.check-column input:checked').each(function(i, el){
                                                jQuery(el).closest('tr').remove();
                                            });
                                        }
                                        return false;
                                    });

                                });
                            </script>
                        </td>
                    </tr>
                    <?php
                    return ob_get_clean();
                }

                /* Add configuration fields */
                function init_form_fields()
                {
                    $this->form_fields = array(
                        'enabled'               => array(
                            'title'             => __('Enable/Disable', 'woocommerce-dpdfrance'),
                            'type'              => 'checkbox',
                            'label'             => __('Enable DPD CLASSIC delivery', 'woocommerce-dpdfrance'),
                            'default'           => 'yes'
                        ),
                        'title'                 => array(
                            'title'             => __('Title', 'woocommerce-dpdfrance'),
                            'type'              => 'text',
                            'description'       => __('This controls the title which the user sees during checkout.', 'woocommerce-dpdfrance'),
                            'default'           => __('Delivery at workplace', 'woocommerce-dpdfrance')
                        ),
                        'depot_code'            => array(
                            'title'             => __('Your DPD Depot code', 'woocommerce-dpdfrance'),
                            'type'              => 'text',
                            'description'       => __('Don\'t hesitate to contact your DPD sales representative to obtain this data', 'woocommerce-dpdfrance'),
                        ),
                        'shipper_code'          => array(
                            'title'             => __('Your DPD CLASSIC contract number', 'woocommerce-dpdfrance'),
                            'type'              => 'text',
                            'description'       => __('Don\'t hesitate to contact your DPD sales representative to obtain this data', 'woocommerce-dpdfrance'),
                        ),
                        'tax_status'            => array(
                            'title'             => __( 'Tax Status', 'woocommerce-dpdfrance' ),
                            'type'              => 'select',
                            'default'           => 'taxable',
                            'options'           => array(
                                'taxable'       => __( 'Taxable', 'woocommerce-dpdfrance' ),
                                'none'          => __( 'None', 'woocommerce-dpdfrance' ),
                            ),
                        ),
                        'handling_fee'          => array(
                            'title'             => __( 'Handling Fee', 'woocommerce-dpdfrance' ),
                            'description'       => __( 'Fee, excluding tax. Leave blank to disable.', 'woocommerce-dpdfrance' ),
                            'type'              => 'price',
                            'default'           => '0.00',
                            'desc_tip'          => true,
                        ),
                        'supp_montagne'         => array(
                            'title'             => __( 'Mountain area Overcost', 'woocommerce-dpdfrance' ),
                            'description'       => __( 'Fee, excluding tax. Enter -1 to disable delivery to these areas.', 'woocommerce-dpdfrance' ),
                            'type'              => 'price',
                            'default'           => '0.00',
                            'desc_tip'          => true,
                        ),
                        'supp_iles'             => array(
                            'title'             => __( 'Coastal Islands & Corsica Overcost', 'woocommerce-dpdfrance' ),
                            'description'       => __( 'Fee, excluding tax. Enter -1 to disable delivery to these areas.', 'woocommerce-dpdfrance' ),
                            'type'              => 'price',
                            'default'           => '0.00',
                            'desc_tip'          => true,
                        ),
                        'franco'                => array(
                            'title'             => __( 'Offer free shipping for orders above this amount', 'woocommerce-dpdfrance' ),
                            'description'       => __( 'Disable this feature with a zero value.', 'woocommerce-dpdfrance' ),
                            'type'              => 'price',
                            'default'           => '0.00',
                            'desc_tip'          => true,
                        ),
                        'max_weight'            => array(
                            'title'             => __( 'Disable shipping method for orders above this weight', 'woocommerce-dpdfrance' ).' ('.get_option('woocommerce_weight_unit').')',
                            'description'       => __( 'Disable this feature with a zero value.', 'woocommerce-dpdfrance' ),
                            'type'              => 'text',
                            'default'           => '30.00',
                            'desc_tip'          => true,
                        ),
                        'coupon_freeshipping'   => array(
                            'title'             => __( 'Enable free shipping coupons mangement', 'woocommerce-dpdfrance' ),
                            'description'       => __( 'If the customer applies a valid coupon, shipping will be free.', 'woocommerce-dpdfrance' ),
                            'desc_tip'          => true,
                            'type'              => 'select',
                            'default'           => 'enabled',
                            'options'           => array(
                                'enabled'       => __( 'Enabled', 'woocommerce-dpdfrance' ),
                                'disabled'      => __( 'Disabled', 'woocommerce-dpdfrance' ),
                            ),
                        ),
                        'classes_to_exclude'    => array(
                            'title'             => __( 'Shipping classes to exclude', 'woocommerce-dpdfrance' ),
                            'description'       => __( 'Specify the shipping classes identifiers to exclude, each one separated by a comma ","', 'woocommerce-dpdfrance' ),
                            'desc_tip'          => true,
                            'type'              => 'text',
                        ),
                        'zones_table'           => array(
                            'type'              => 'zones_table',
                        ),
                        'table_rates_table'     => array(
                            'type'              => 'table_rates_table'
                        ),
                    );
                }

                /* Process and save submitted zones */
                function process_zones()
                {
                    // Save the zones
                    $zone_id = array();
                    $zone_name = array();
                    $zone_country = array();
                    $zone_include = array();
                    $zone_exclude = array();
                    $zone_type = array();
                    $zone_enabled = array();

                    $zones = array();

                    if ( isset( $_POST[ $this->id . '_zone_id'] ) ) $zone_id = array_map( 'wc_clean', $_POST[ $this->id . '_zone_id'] );
                    if ( isset( $_POST[ $this->id . '_zone_name'] ) ) $zone_name = array_map( 'wc_clean', $_POST[ $this->id . '_zone_name'] );
                    if ( isset( $_POST[ $this->id . '_zone_country'] ) ) $zone_country =  $_POST[ $this->id . '_zone_country'];
                    if ( isset( $_POST[ $this->id . '_zone_include'] ) ) $zone_include = array_map( 'wc_clean', $_POST[ $this->id . '_zone_include'] );
                    if ( isset( $_POST[ $this->id . '_zone_exclude'] ) ) $zone_exclude = array_map( 'wc_clean', $_POST[ $this->id . '_zone_exclude'] );
                    if ( isset( $_POST[ $this->id . '_zone_type'] ) ) $zone_type = array_map( 'wc_clean', $_POST[ $this->id . '_zone_type'] );
                    if ( isset( $_POST[ $this->id . '_zone_enabled'] ) ) $zone_enabled = array_map( 'wc_clean', $_POST[ $this->id . '_zone_enabled'] );

                    // Get max key
                    $values = $zone_id;
                    ksort( $values );
                    $value = end( $values );
                    $key = key( $values );

                    for ( $i = 0; $i <= $key; $i++ ) {
                        if ( isset( $zone_id[ $i ] )
                            && ! empty( $zone_name[ $i ] )
                            && ! empty( $zone_country[ $i ] )
                            && isset( $zone_include[ $i ] )
                            && isset( $zone_exclude[ $i ] )
                            && isset( $zone_type[ $i ] )
                            && isset( $zone_enabled[ $i ] ) ){

                            // Add to zones array
                            $zones[] = array(
                                'id' => $zone_id[ $i ],
                                'name' => $zone_name[ $i ],
                                'country' => $zone_country[ $i ],
                                'include' => $zone_include[ $i ],
                                'exclude' => $zone_exclude[ $i ],
                                'type' => $zone_type[ $i ],
                                'enabled' => $zone_enabled[ $i ],
                            );
                        }
                    }

                    if ( (!empty($zone_id[$key]))
                        && ($zone_id[$key] > $this->last_zone_id)
                        && (is_numeric($zone_id[$key]))) {
                        $highest_zone_id = $zone_id[$key];
                        update_option( $this->last_zone_id_option, $highest_zone_id );
                    }

                    update_option( $this->zones_option, $zones );

                    $this->get_zones();

                }

                /* Retrieves zones array from database */
                function get_zones()
                {
                    $this->zones = array_filter( (array) get_option( $this->zones_option ) );
                }

                /* Retrieves last zone id from database */
                function get_last_zone_id()
                {
                    $this->last_zone_id = (int)get_option( $this->last_zone_id_option );
                }

                /* Process and save submitted table_rates */
                function process_table_rates()
                {
                    // Save the rates
                    $table_rate_id = array();
                    $table_rate_zone = array();
                    $table_rate_basis = array();
                    $table_rate_min = array();
                    $table_rate_max = array();
                    $table_rate_cost = array();
                    $table_rate_enabled = array();

                    $table_rates = array();

                    if ( isset( $_POST[ $this->id . '_table_rate_id'] ) ) $table_rate_id = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_id'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_zone'] ) ) $table_rate_zone = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_zone'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_basis'] ) ) $table_rate_basis = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_basis'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_min'] ) )   $table_rate_min   = array_map( 'stripslashes', $_POST[ $this->id . '_table_rate_min'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_max'] ) )   $table_rate_max   = array_map( 'stripslashes', $_POST[ $this->id . '_table_rate_max'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_cost'] ) )  $table_rate_cost  = array_map( 'stripslashes', $_POST[ $this->id . '_table_rate_cost'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_enabled'] ) ) $table_rate_enabled = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_enabled'] );

                    // Get max key
                    $values = $table_rate_id;
                    ksort( $values );
                    $value = end( $values );
                    $key = key( $values );

                    for ( $i = 0; $i <= $key; $i++ ) {
                        if ( isset( $table_rate_id[ $i ] )
                            && isset( $table_rate_zone[ $i ] )
                            && isset( $table_rate_basis[ $i ] )
                            && isset( $table_rate_min[ $i ] )
                            && isset( $table_rate_max[ $i ] )
                            && isset( $table_rate_cost[ $i ] )
                            && isset( $table_rate_enabled[ $i ] ) ) {

                            $table_rate_cost[ $i ] = wc_format_decimal( $table_rate_cost[$i] );

                            // Add table_rates to array
                            $table_rates[] = array(
                                'id' => $table_rate_id[ $i ],
                                'zone' => $table_rate_zone[ $i ],
                                'basis' => $table_rate_basis[ $i ],
                                'min' => $table_rate_min[ $i ],
                                'max' => $table_rate_max[ $i ],
                                'cost' => $table_rate_cost[ $i ],
                                'enabled' => $table_rate_enabled[ $i ],
                            );
                        }
                    }

                    if ( (!empty($table_rate_id[$key]))
                        && ($table_rate_id[$key] > $this->last_table_rate_id)
                        && (is_numeric($table_rate_id[$key]))) {
                        $highest_table_rate_id = $table_rate_id[$key];
                        update_option( $this->last_table_rate_id_option, $highest_table_rate_id );
                    }

                    update_option( $this->table_rate_option, $table_rates );

                    $this->get_table_rates();

                }

                /* Retrieves table_rates array from database */
                function get_table_rates()
                {
                    $this->table_rates = array_filter( (array) get_option( $this->table_rate_option ) );
                }

                /* Retrieves last table_rate id from database */
                function get_last_table_rate_id()
                {
                    $this->last_table_rate_id = (int)get_option( $this->last_table_rate_id_option );
                }

                /* Retrieves available zone ids for supplied shipping address */
                function get_available_zones($package)
                {

                    $destination_country = $package['destination']['country'];
                    $available_zones = array();

                    foreach ($this->zones as $zone):
                        if ( !empty($zone['country']) && in_array($destination_country, $zone['country']) ) {
                            $available_zones[] = $zone['id'];
                        }
                    endforeach;

                    if (empty($available_zones)) {
                        $found = false;
                        foreach (WC()->countries->get_shipping_countries() as $id => $value):
                            if ($destination_country == $id) {
                                $found = true;
                            }
                        endforeach;
                        if ($found) {
                            $available_zones[] = '0'; // "Everywhere else" zone
                        }
                    }

                    return $available_zones;
                }

                /* Retrieves available table_rates for cart and supplied shipping addresss */
                function get_available_table_rates($package)
                {
                    global $woocommerce;

                    $available_zones = $this->get_available_zones($package);
                    $available_table_rates = array();
                    $weight = $this->get_cart_weight($package);
                    if (WC()->version >= '3.2.0') {
                        $cart_taxes = array_sum($woocommerce->cart->get_cart_contents_taxes());
                    } else {
                        $cart_taxes = array_sum($woocommerce->cart->taxes);
                    }
                    $total = $woocommerce->cart->cart_contents_total + $cart_taxes;

                    foreach ($this->table_rates as $table_rate) {
                        // Is table_rate for an available zone?
                        $zone_pass = (in_array($table_rate['zone'], $available_zones));

                        // Is table_rate valid for basket weight?
                        if ($table_rate['basis'] == 'weight') {
                            $weight_pass = (($weight >= $table_rate['min']) && ($this->is_less_than($weight, $table_rate['max'])));
                        } else {
                            $weight_pass = true;
                        }

                        // Is table_rate valid for basket total?
                        if ($table_rate['basis'] == 'price') {
                            $total_pass = (($total >= $table_rate['min']) && ($this->is_less_than($total, $table_rate['max'])));
                        } else {
                            $total_pass = true;
                        }

                        // Accept table_rate if passes all tests
                        if ($zone_pass && $weight_pass && $total_pass) {
                            $available_table_rates[] = $table_rate;
                        }
                    }
                    return $available_table_rates;
                }

                /* Retrieves total cart weight */
                function get_cart_weight($package) {
                    $total = 0;
                    foreach ( $package['contents'] as $cart_item_key => $cart_item ) {
                        $product = $cart_item['data'];
                        $total += ((float)$product->get_weight() * $cart_item['quantity']);
                    }
                    return $total;
                }

                /* Return true if value less than max, incl. "*" */
                function is_less_than($value, $max)
                {
                    if ($max == '*')
                        return true;
                    else
                        return ($value <= $max);
                }

                /* Retrieves an array item by searching for an id value */
                function find_by_id($array, $id)
                {
                    foreach($array as $a):
                        if ($a['id'] == $id)
                            return $a;
                    endforeach;
                    return false;
                }

                /* Retrieves cheapest rate from a list of table_rates. */
                function pick_cheapest_table_rate($table_rates)
                {
                    $cheapest = false;
                    foreach ($table_rates as $table_rate):
                        if ($cheapest == false)
                            $cheapest = $table_rate;
                        else{
                            if ($table_rate['cost'] < $cheapest['cost'])
                                $cheapest = $table_rate;
                        }
                    endforeach;
                    return $cheapest;
                }

                /* Calculate shipping cost. This is called by WooCommerce */
                public function calculate_shipping($package = array())
                {
                    global $woocommerce;

                    $available_table_rates = $this->get_available_table_rates($package);
                    $table_rate = $this->pick_cheapest_table_rate($available_table_rates);

                    if (!isset($this->settings['tax_status']) || $this->settings['tax_status'] == 'none')
                        $tax = false;
                    else
                        $tax = '';

                    if ($table_rate != false)
                    {
                        /* Handling fee calculation */
                        if (!empty($this->settings['handling_fee']) && $this->settings['handling_fee'] > 0) {
                            $cost = $table_rate['cost'] + $this->settings['handling_fee'];
                        } else {
                            $cost = $table_rate['cost'];
                        }

                        /* Overcost calculation */
                        $cdts = dpdfrance_classic_getOrderShippingCost($package['destination']['postcode'], $cost);

                        /* Coupon override */
                        if ($this->settings['coupon_freeshipping'] == 'enabled') {
                            if ($coupons = WC()->cart->get_coupons()) {
                                foreach ($coupons as $code => $coupon) {
                                    if ($coupon->is_valid() && $coupon->get_free_shipping()) {
                                        $cdts['cost'] = 0;
                                    }
                                }
                            }
                        }

                        /* Franco override */
                        if (!empty($this->settings['franco']) && $this->settings['franco'] > 0) {
                            if (WC()->version >= '3.2.0') {
                                $cart_taxes = array_sum($woocommerce->cart->get_cart_contents_taxes());
                            } else {
                                $cart_taxes = array_sum($woocommerce->cart->taxes);
                            }
                            $total = $woocommerce->cart->cart_contents_total + $cart_taxes;
                            if ($total >= $this->settings['franco']) {
                                $cdts['cost'] = 0;
                            }
                        }

                        /* Max weight override */
                        if (!empty($this->settings['max_weight']) && $this->settings['max_weight'] > 0) {
                            $weight = $this->get_cart_weight($package);
                            if ($weight >= $this->settings['max_weight']) {
                                $cdts['excl'] = 2;
                            }
                        }

                        /* Exclude shipping classes */
                        if (!empty($this->settings['classes_to_exclude'])) {
                            $cart_shipping_classes = dpdfrance_classic_find_shipping_classes($package);
                            $classes_to_exclude = explode(',', $this->settings['classes_to_exclude']);
                            if (array_intersect($classes_to_exclude, $cart_shipping_classes)) {
                                $cdts['excl'] = 3;
                            }
                        }

                        /* Register the rate */
                        $rate = array(
                            'id' => $this->id,
                            'label' => $this->title.($cdts['cost'] == 0 ? ': '.__('free', 'woocommerce-dpdfrance') : ''),
                            'cost' => $cdts['cost'],
                            'taxes' => $tax,
                            'calc_tax' => 'per_order'
                        );
                        if ($cdts['excl'] == 0)
                            $this->add_rate($rate);
                    }
                }
            }
        }
        /* End of class */

        /* Add shipping mehod */
        function add_dpdfrance_classic_delivery_method($methods) {
            $methods[] = 'WC_dpdfrance_classic';
            return $methods;
        }

        /* Hide shipping cost 0.00 when free shipping */
        add_filter( 'woocommerce_cart_shipping_method_full_label', function( $label, $method ) {
            if ($method->id == 'dpdfrance_classic' && $method->cost <= 0 ) {
                $label = $method->get_label();
            }
            return $label;}, 10, 2 );
        add_filter('woocommerce_shipping_methods', 'add_dpdfrance_classic_delivery_method');
    }

    /* Load Translation files */
    function woocommerce_dpdfrance_classic_load_plugin_textdomain()
    {
        load_plugin_textdomain( 'woocommerce-dpdfrance', FALSE, basename(dirname(__FILE__)) . '/../../../languages/plugins/');
    }
    add_action( 'init', 'woocommerce_dpdfrance_classic_load_plugin_textdomain' );

    /* Island and Mountain zones overcost calculation functions */
    function dpdfrance_classic_getOrderShippingCost($postcode, $cost = 0)
    {
        global $woocommerce;

        $excl = 0;
        $cfg = new WC_dpdfrance_classic();
        $supp_iles = $cfg->settings['supp_iles'];
        $supp_montagne = $cfg->settings['supp_montagne'];

        $zone_iles = array('17111','17123','17190','17310','17370','17410','17480','17550','17580','17590','17630','17650','17670','17740','17840','17880','17940','22870','29242','29253','29259','29980','29990','56360','56590','56780','56840','85350');
        $zone_montagne = array('04120','04130','04140','04160','04170','04200','04240','04260','04300','04310','04330','04360','04370','04400','04510','04530','04600','04700','04850','05100','05110','05120','05130','05150','05160','05170','05200','05220','05240','05250','05260','05290','05300','05310','05320','05330','05340','05350','05400','05460','05470','05500','05560','05600','05700','05800','06140','06380','06390','06410','06420','06430','06450','06470','06530','06540','06620','06710','06750','06910','09110','09140','09300','09460','25120','25140','25240','25370','25450','25500','25650','30570','31110','38112','38114','38142','38190','38250','38350','38380','38410','38580','38660','38700','38750','38860','38880','39220','39310','39400','63113','63210','63240','63610','63660','63690','63840','63850','64440','64490','64560','64570','65110','65120','65170','65200','65240','65400','65510','65710','66210','66760','66800','68140','68610','68650','73110','73120','73130','73140','73150','73160','73170','73190','73210','73220','73230','73250','73260','73270','73300','73320','73340','73350','73390','73400','73440','73450','73460','73470','73500','73530','73550','73590','73600','73620','73630','73640','73710','73720','73870','74110','74120','74170','74220','74230','74260','74310','74340','74350','74360','74390','74400','74420','74430','74440','74450','74470','74480','74660','74740','74920','83111','83440','83530','83560','83630','83690','83830','83840','84390','88310','88340','88370','88400','90200');

        if (substr($postcode, 0, 2) == '20' || in_array($postcode, $zone_iles)) {
            if ($supp_iles >= 0)
                $cost = $cost + $cfg->settings['supp_iles'];
            else
                $excl = 1;
        }

        if (in_array($postcode, $zone_montagne)) {
            if ($supp_montagne >= 0)
                $cost = $cost + $cfg->settings['supp_montagne'];
            else
                $excl = 2;
        }

        return array('excl' => $excl, 'cost' => $cost);
    }

    /* Get the shipping classes inside cart*/
    function dpdfrance_classic_find_shipping_classes($package) {
        $found_shipping_classes = array();
        foreach ( $package['contents'] as $item_id => $values ) {
            if ( $values['data']->needs_shipping() ) {
                $found_shipping_classes[$item_id] = $values['data']->get_shipping_class();
            }
        }
        return $found_shipping_classes;
    }
}