<?php
/* Exit if accessed directly */
if (!defined('ABSPATH'))
    exit;

/* Check if WooCommerce is active */
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
if ( is_plugin_active( 'woocommerce/woocommerce.php' ) ) {

    /* Init plugin */
    function woocommerce_dpdfrance_relais_init()
    {
        if (!class_exists('WC_dpdfrance_relais'))
        {
            /* Main class */
            class WC_dpdfrance_relais extends WC_Shipping_Method
            {
                function __construct() {
                    $this->id                           = 'dpdfrance_relais';
                    $this->table_rate_option            = $this->id.'_table_rates';
                    $this->last_table_rate_id_option    = $this->id.'_last_table_rate_id';
                    $this->zones_option                 = $this->id.'_zones';
                    $this->last_zone_id_option          = $this->id.'_last_zone_id';
                    $this->method_title                 = __('DPD Relais (France only)', 'woocommerce-dpdfrance');
                    $this->method_description           = __('DPD Relais delivery plugin for WooCommerce - France vendors only', 'woocommerce-dpdfrance' );
                    $this->get_zones();
                    $this->get_last_zone_id();
                    $this->get_table_rates();
                    $this->get_last_table_rate_id();

                    $this->init();

                    foreach ($this->settings as $key => $val)
                        $this->$key = $val;
                }

                function init()
                {
                    // Load the settings API
                    $this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
                    $this->init_settings(); // This is part of the settings API. Loads settings you previously init.

                    $this->enabled = $this->settings['enabled'];
                    $this->title = $this->settings['title']; // Shown in drop down and admin order screen

                    // Save settings in admin if you have any defined
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_zones' ) );
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_table_rates' ) );
                }

                /* Generates dropdown menus */
                function generate_options()
                {
                    $option_arr = array();

                    /* Zones */
                    foreach($this->zones as $option):
                        $option_arr['table_rate_zone'][esc_attr($option['id'])] = esc_js($option['name']);
                    endforeach;

                    $option_arr['table_rate_zone']['0'] = __( 'Everywhere Else', 'woocommerce-dpdfrance');

                    /* Countries */
                    foreach (WC()->countries->get_shipping_countries() as $id => $value) :
                        $option_arr['country'][esc_attr($id)] = esc_js($value);
                    endforeach;

                    /* Table rates */
                    $option_arr['rate_basis']['weight'] = sprintf(__( 'Weight (%s)', 'woocommerce-dpdfrance' ), get_option('woocommerce_weight_unit'));
                    $option_arr['rate_basis']['price'] = sprintf(__( 'Total (%s)', 'woocommerce-dpdfrance' ), get_woocommerce_currency_symbol());

                    return $option_arr;
                }

                /* Generates HTML code for top of configuration page */
                function admin_options() {
                ?>
                    <script type="text/javascript">

                        var options = <?php echo json_encode($this->generate_options()); ?>;

                        function generateSelectOptionsHtml(options, selected) {
                            var html;
                            var selectedHtml;

                            for (var key in options) {
                                var value = options[key];

                                if (selected instanceof Array) {
                                    if (selected.indexOf(key) != -1) {
                                        selectedHtml = ' selected="selected"';
                                    } else {
                                        selectedHtml = '';
                                    }
                                } else {
                                    if (key == selected) {
                                        selectedHtml = ' selected="selected"';
                                    } else {
                                        selectedHtml = '';
                                    }
                                }

                                html += '<option value="' + key +'"' + selectedHtml + '>' + value + '</option>';
                            }

                            return html;
                        }
                    </script>
                    <style>
                        .debug-col {
                            display: none;
                        }
                        table.shippingrows tr th {
                            padding-left: 10px;
                        }
                        .zone td {
                            vertical-align: top;
                        }
                        .zone textarea {
                            width: 100%;
                        }

                    </style>
                    <h2><?php _e('DPD Relais (France only)', 'woocommerce-dpdfrance'); ?></h2>
                    <div class="updated woocommerce-message">
                        <p><strong><?php _e('Welcome to DPD.', 'woocommerce-dpdfrance'); ?></strong> <?php _e('The plugin documentation is available here :', 'woocommerce-dpdfrance'); ?> <a target="_blank" href="<?php echo OS_DPDFRANCE_ROOT_URL;?>/assets/docs/readme_dpdfrance_woocommerce.pdf"><img src="<?php echo OS_DPDFRANCE_ROOT_URL;?>/assets/img/admin/pdf.png"/></a></p>
                    </div>
                    <table class="form-table">
                    <?php $this->generate_settings_html(); ?>
                    </table>

                <?php
                }

                /* Generates HTML code for zone settings table */
                function generate_zones_table_html() {
                    ob_start();
                    ?>
                    <tr valign="top">
                        <th scope="row" class="titledesc"><?php _e( 'Shipping Zones', 'woocommerce-dpdfrance' ); ?></th>
                        <td class="forminp" id="<?php echo $this->id; ?>_zones">
                            <p style="padding-bottom: 10px;"><?php _e( 'After adding a shipping zone, hit "Save changes" so that it appears as an option in the table rate section.', 'woocommerce-dpdfrance' ); ?></p>
                            <table class="shippingrows widefat" cellspacing="0">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:100%;">
                                <thead>
                                    <tr>
                                        <th class="check-column"><input type="checkbox"></th>
                                        <!--<th class="debug-col"><?php _e( 'ID', 'woocommerce-dpdfrance' ); ?></th>-->
                                        <th><?php _e( 'Name', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Shipping zone name, will appear in table rates table.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Countries', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Add one or more countries that are part of this shipping zone.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                    </tr>
                                </thead>
                                <tfoot>
                                    <tr>
                                        <th colspan="3"><a href="#" class="add button"><?php _e( 'Add Shipping Zone', 'woocommerce-dpdfrance' ); ?></a> <a href="#" class="remove button"><?php _e( 'Delete Selected Zones', 'woocommerce-dpdfrance' ); ?></a></th>
                                    </tr>
                                </tfoot>
                                <tbody class="zones">
                                    <tr class="zone">
                                        <th></th>
                                        <!--<td class="debug-col">0</td>-->
                                        <td><div style="width: 200px;"><?php _e( 'Default Zone (everywhere else)', 'woocommerce-dpdfrance' ); ?></div></td>
                                        <td><em><?php _e( 'All allowed countries', 'woocommerce-dpdfrance' ); ?></em></td>
                                    </tr>
                                </tbody>
                            </table>
                            <script type="text/javascript">

                                var lastZoneId = <?php echo $this->last_zone_id; ?>;

                                <?php
                                foreach ($this->zones as $zone):
                                    $js_array = json_encode($zone);
                                    echo "jQuery('#{$this->id}_zones table tbody tr:last').before(addZoneRowHtml(false, {$js_array}));\n";
                                endforeach;
                                ?>

                                function addZoneRowHtml(isNew, rowArr) {

                                    if (isNew) {
                                        lastZoneId++;
                                        rowArr = {};
                                        rowArr['id'] = lastZoneId;
                                        rowArr['name'] = '';
                                        rowArr['country'] = '';
                                        rowArr['type'] = 'country';
                                        rowArr['include'] = '';
                                        rowArr['exclude'] = '';
                                        rowArr['enabled'] = '1';
                                    }

                                    var size = jQuery('#<?php echo $this->id; ?>_zones tbody .zone').size();
                                    var html = '\
                                            <tr class="zone">\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_id[' + size + ']" value="' + rowArr['id'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_type[' + size + ']" value="' + rowArr['type'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_include[' + size + ']" value="' + rowArr['include'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_exclude[' + size + ']" value="' + rowArr['exclude'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_zone_enabled[' + size + ']" value="' + rowArr['enabled'] + '" />\
                                                <th class="check-column"><input type="checkbox" name="select" /></th>\
                                                <!--<td class="debug-col">\
                                                    ' + rowArr['id'] + '\
                                                </td>-->\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_zone_name[' + size + ']" value="' + rowArr['name'] + '" size="30" placeholder="" />\
                                                </td>\
                                                <td style="overflow:visible;">\
                                                    <select multiple="multiple" name="<?php echo $this->id; ?>_zone_country[' + size + '][]" class="multiselect chosen_select">\
                                                        ' + generateSelectOptionsHtml(options['country'], rowArr['country']) + '\
                                                    </select>\
                                                </td>\
                                            </tr>';
                                    return html;
                                }

                                jQuery(function() {

                                    jQuery('#<?php echo $this->id; ?>_zones').on( 'click', 'a.add', function(){

                                        jQuery('#<?php echo $this->id; ?>_zones table tbody tr:last').before(addZoneRowHtml(true, false));

                                        if (jQuery().chosen) {
                                            jQuery("select.chosen_select").chosen({
                                                width: '350px',
                                                disable_search_threshold: 5
                                            });
                                        } else {
                                            jQuery("select.chosen_select").select2();
                                        }

                                        return false;
                                    });

                                    // Remove row
                                    jQuery('#<?php echo $this->id; ?>_zones').on( 'click', 'a.remove', function(){

                                        var answer = confirm("<?php _e( 'Delete the selected zones?', 'woocommerce-dpdfrance' ); ?>");
                                        if (answer) {
                                            jQuery('#<?php echo $this->id; ?>_zones table tbody tr th.check-column input:checked').each(function(i, el){
                                                jQuery(el).closest('tr').remove();
                                            });
                                        }
                                        return false;
                                    });

                                });
                            </script>
                        </td>
                    </tr>
                    <?php
                    return ob_get_clean();
                }

                /* Generates HTML for table_rate settings table */
                function generate_table_rates_table_html() {
                    ob_start();
                    ?>
                    <tr valign="top">
                        <th scope="row" class="titledesc"><?php _e( 'Table Rates', 'woocommerce-dpdfrance' ); ?></th>
                        <td class="forminp" id="<?php echo $this->id; ?>_table_rates">
                            <table class="shippingrows widefat" cellspacing="0">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:0%">
                                <col style="width:100%;">
                                <thead>
                                    <tr>
                                        <th class="check-column"><input type="checkbox"></th>
                                        <th class="debug-col"><?php _e( 'ID', 'woocommerce-dpdfrance' ); ?></th>
                                        <th><?php _e( 'Zone', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Shipping zone, as defined in Shipping Zones table.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Condition', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Choose which metric to base your table rate on.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Min', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Minimum, in decimal format. Inclusive.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Max', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Maximum, in decimal format. Inclusive. To impose no upper limit, use *".', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                        <th><?php _e( 'Cost', 'woocommerce-dpdfrance' ); ?> <a class="tips" data-tip="<?php _e( 'Cost, excluding tax.', 'woocommerce-dpdfrance' ); ?>">[?]</a></th>
                                    </tr>
                                </thead>
                                <tfoot>
                                    <tr>
                                        <th colspan="7"><a href="#" class="add button"><?php _e( 'Add Table Rate', 'woocommerce-dpdfrance' ); ?></a> <a href="#" class="remove button"><?php _e( 'Delete Selected Rates', 'woocommerce-dpdfrance' ); ?></a></th>
                                    </tr>
                                </tfoot>
                                <tbody class="table_rates">

                                </tbody>
                            </table>
                            <script type="text/javascript">

                                var lastTableRateId = <?php echo $this->last_table_rate_id; ?>;

                                <?php
                                foreach ($this->table_rates as $table_rate):
                                    $js_array = json_encode($table_rate);
                                    echo "jQuery(addTableRateRowHtml(false, {$js_array})).appendTo('#{$this->id}_table_rates table tbody');\n";
                                endforeach;
                                ?>

                                function addTableRateRowHtml(isNew, rowArr) {

                                    if (isNew) {
                                        lastTableRateId++;
                                        rowArr = {};
                                        rowArr['id'] = lastTableRateId;
                                        rowArr['zone'] = '<?php echo (!empty($this->zones[0]['id'])) ? $this->zones[0]['id'] : 0; ?>';
                                        rowArr['basis'] = 'weight';
                                        rowArr['min'] = '0';
                                        rowArr['max'] = '*';
                                        rowArr['cost'] = '0';
                                        rowArr['enabled'] = '1';
                                    }

                                    var size = jQuery('#<?php echo $this->id; ?>_table_rates tbody .table_rate').size();
                                    var html = '\
                                            <tr class="table_rate">\
                                                <input type="hidden" name="<?php echo $this->id; ?>_table_rate_id[' + size + ']" value="' + rowArr['id'] + '" />\
                                                <input type="hidden" name="<?php echo $this->id; ?>_table_rate_enabled[' + size + ']" value="' + rowArr['enabled'] + '" />\
                                                <th class="check-column"><input type="checkbox" name="select" /></th>\
                                                <td class="debug-col">\
                                                    ' + rowArr['id'] + '\
                                                </td>\
                                                <td>\
                                                    <select name="<?php echo $this->id; ?>_table_rate_zone[' + size + ']">\
                                                        ' + generateSelectOptionsHtml(options['table_rate_zone'], rowArr['zone']) + '\
                                                    </select>\
                                                </td>\
                                                <td>\
                                                    <select name="<?php echo $this->id; ?>_table_rate_basis[' + size + ']">\
                                                        ' + generateSelectOptionsHtml(options['rate_basis'], rowArr['basis']) + '\
                                                    </select>\
                                                </td>\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_table_rate_min[' + size + ']" value="' + rowArr['min'] + '" placeholder="0" size="4" />\
                                                </td>\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_table_rate_max[' + size + ']" value="' + rowArr['max'] + '" placeholder="*" size="4" />\
                                                </td>\
                                                <td>\
                                                    <input type="text" name="<?php echo $this->id; ?>_table_rate_cost[' + size + ']" value="' + rowArr['cost'] + '" placeholder="<?php echo wc_format_localized_price( 0 ); ?>" size="4" class="wc_input_price" />\
                                                </td>\
                                            </tr>';
                                    return html;
                                }

                                jQuery(function() {

                                    jQuery('#<?php echo $this->id; ?>_table_rates').on( 'click', 'a.add', function(){

                                        jQuery(addTableRateRowHtml(true, false)).appendTo('#<?php echo $this->id; ?>_table_rates table tbody');

                                        return false;
                                    });

                                    // Remove row
                                    jQuery('#<?php echo $this->id; ?>_table_rates').on( 'click', 'a.remove', function(){
                                        var answer = confirm("<?php _e( 'Delete the selected rates?', 'woocommerce-dpdfrance' ); ?>");
                                        if (answer) {
                                            jQuery('#<?php echo $this->id; ?>_table_rates table tbody tr th.check-column input:checked').each(function(i, el){
                                                jQuery(el).closest('tr').remove();
                                            });
                                        }
                                        return false;
                                    });

                                });
                            </script>
                        </td>
                    </tr>
                    <?php
                    return ob_get_clean();
                }

                /* Add configuration fields */
                function init_form_fields()
                {
                    $this->form_fields = array(
                        'enabled'                      => array(
                            'title'                    => __('Enable/Disable', 'woocommerce-dpdfrance'),
                            'type'                     => 'checkbox',
                            'label'                    => __('Enable DPD Relais delivery', 'woocommerce-dpdfrance'),
                            'default'                  => 'yes'
                        ),
                        'title'                        => array(
                            'title'                    => __('Title', 'woocommerce-dpdfrance'),
                            'type'                     => 'text',
                            'description'              => __('This controls the title which the user sees during checkout.', 'woocommerce-dpdfrance'),
                            'default'                  => __('Pickup point delivery', 'woocommerce-dpdfrance')
                        ),
                        'depot_code'                   => array(
                            'title'                    => __('Your DPD Depot code', 'woocommerce-dpdfrance'),
                            'type'                     => 'text',
                            'description'              => __('Don\'t hesitate to contact your DPD sales representative to obtain this data', 'woocommerce-dpdfrance'),
                        ),
                        'shipper_code'                 => array(
                            'title'                    => __('Your DPD Relais contract number', 'woocommerce-dpdfrance'),
                            'type'                     => 'text',
                            'description'              => __('Don\'t hesitate to contact your DPD sales representative to obtain this data', 'woocommerce-dpdfrance'),
                        ),
                        'dpdfrance_relais_MerchantID'  => array(
                            'title'                    => __('Merchant ID', 'woocommerce-dpdfrance'),
                            'type'                     => 'text',
                            'description'              => __('Please enter the merchant ID provided by DPD.', 'woocommerce-dpdfrance'),
                            'default'                  => ''
                        ),
                        'dpdfrance_relais_SecurityKey' => array(
                            'title'                    => __('Security Key', 'woocommerce-dpdfrance'),
                            'type'                     => 'text',
                            'description'              => __('Please enter the secuity key provided by DPD.', 'woocommerce-dpdfrance'),
                            'default'                  => ''
                        ),
                        'dpdfrance_relais_MyPudoURL'   => array(
                            'title'                    => __('DPD Relais Webservice URL', 'woocommerce-dpdfrance'),
                            'type'                     => 'text',
                            'description'              => __('Please enter the DPD Relais WebService URL.', 'woocommerce-dpdfrance'),
                            'default'                  => 'http://mypudo.pickup-services.com/mypudo/mypudo.asmx?WSDL'
                        ),
                        'tax_status'                   => array(
                            'title'                    => __( 'Tax Status', 'woocommerce-dpdfrance' ),
                            'type'                     => 'select',
                            'default'                  => 'taxable',
                            'options'                  => array(
                                'taxable'              => __( 'Taxable', 'woocommerce-dpdfrance' ),
                                'none'                 => __( 'None', 'woocommerce-dpdfrance' ),
                            ),
                        ),
                        'handling_fee'                 => array(
                            'title'                    => __( 'Handling Fee', 'woocommerce-dpdfrance' ),
                            'description'              => __( 'Fee, excluding tax. Leave blank to disable.', 'woocommerce-dpdfrance' ),
                            'type'                     => 'price',
                            'default'                  => '0.00',
                            'desc_tip'                 => true,
                        ),
                        'supp_montagne'                => array(
                            'title'                    => __( 'Mountain area Overcost', 'woocommerce-dpdfrance' ),
                            'description'              => __( 'Fee, excluding tax. Enter -1 to disable delivery to these areas.', 'woocommerce-dpdfrance' ),
                            'type'                     => 'price',
                            'default'                  => '0.00',
                            'desc_tip'                 => true,
                        ),
                        'supp_iles'                    => array(
                            'title'                    => __( 'Coastal Islands & Corsica Overcost', 'woocommerce-dpdfrance' ),
                            'description'              => __( 'Fee, excluding tax. Enter -1 to disable delivery to these areas.', 'woocommerce-dpdfrance' ),
                            'type'                     => 'price',
                            'default'                  => '0.00',
                            'desc_tip'                 => true,
                        ),
                        'franco'                       => array(
                            'title'                    => __( 'Offer free shipping for orders above this amount', 'woocommerce-dpdfrance' ),
                            'description'              => __( 'Disable this feature with a zero value.', 'woocommerce-dpdfrance' ),
                            'type'                     => 'price',
                            'default'                  => '0.00',
                            'desc_tip'                 => true,
                        ),
                        'max_weight'                   => array(
                            'title'                    => __( 'Disable shipping method for orders above this weight', 'woocommerce-dpdfrance' ).' ('.get_option('woocommerce_weight_unit').')',
                            'description'              => __( 'Disable this feature with a zero value.', 'woocommerce-dpdfrance' ),
                            'type'                     => 'text',
                            'default'                  => '20.00',
                            'desc_tip'                 => true,
                        ),
                        'coupon_freeshipping'          => array(
                            'title'                    => __( 'Enable free shipping coupons mangement', 'woocommerce-dpdfrance' ),
                            'description'              => __( 'If the customer applies a valid coupon, shipping will be free.', 'woocommerce-dpdfrance' ),
                            'desc_tip'                 => true,
                            'type'                     => 'select',
                            'default'                  => 'enabled',
                            'options'                  => array(
                                'enabled'              => __( 'Enabled', 'woocommerce-dpdfrance' ),
                                'disabled'             => __( 'Disabled', 'woocommerce-dpdfrance' ),
                            ),
                        ),
                        'classes_to_exclude'           => array(
                            'title'                    => __( 'Shipping classes to exclude', 'woocommerce-dpdfrance' ),
                            'description'              => __( 'Specify the shipping classes identifiers to exclude, each one separated by a comma ","', 'woocommerce-dpdfrance' ),
                            'desc_tip'                 => true,
                            'type'                     => 'text',
                        ),
                        'zones_table'                  => array(
                            'type'                     => 'zones_table',
                        ),
                        'table_rates_table'            => array(
                            'type'                     => 'table_rates_table'
                        ),
                    );
                }

                /* Process and save submitted zones */
                function process_zones()
                {
                    // Save the zones
                    $zone_id = array();
                    $zone_name = array();
                    $zone_country = array();
                    $zone_include = array();
                    $zone_exclude = array();
                    $zone_type = array();
                    $zone_enabled = array();

                    $zones = array();

                    if ( isset( $_POST[ $this->id . '_zone_id'] ) ) $zone_id = array_map( 'wc_clean', $_POST[ $this->id . '_zone_id'] );
                    if ( isset( $_POST[ $this->id . '_zone_name'] ) ) $zone_name = array_map( 'wc_clean', $_POST[ $this->id . '_zone_name'] );
                    if ( isset( $_POST[ $this->id . '_zone_country'] ) ) $zone_country =  $_POST[ $this->id . '_zone_country'];
                    if ( isset( $_POST[ $this->id . '_zone_include'] ) ) $zone_include = array_map( 'wc_clean', $_POST[ $this->id . '_zone_include'] );
                    if ( isset( $_POST[ $this->id . '_zone_exclude'] ) ) $zone_exclude = array_map( 'wc_clean', $_POST[ $this->id . '_zone_exclude'] );
                    if ( isset( $_POST[ $this->id . '_zone_type'] ) ) $zone_type = array_map( 'wc_clean', $_POST[ $this->id . '_zone_type'] );
                    if ( isset( $_POST[ $this->id . '_zone_enabled'] ) ) $zone_enabled = array_map( 'wc_clean', $_POST[ $this->id . '_zone_enabled'] );

                    // Get max key
                    $values = $zone_id;
                    ksort( $values );
                    $value = end( $values );
                    $key = key( $values );

                    for ( $i = 0; $i <= $key; $i++ ) {
                        if ( isset( $zone_id[ $i ] )
                            && ! empty( $zone_name[ $i ] )
                            && ! empty( $zone_country[ $i ] )
                            && isset( $zone_include[ $i ] )
                            && isset( $zone_exclude[ $i ] )
                            && isset( $zone_type[ $i ] )
                            && isset( $zone_enabled[ $i ] ) ){

                            // Add to zones array
                            $zones[] = array(
                                'id' => $zone_id[ $i ],
                                'name' => $zone_name[ $i ],
                                'country' => $zone_country[ $i ],
                                'include' => $zone_include[ $i ],
                                'exclude' => $zone_exclude[ $i ],
                                'type' => $zone_type[ $i ],
                                'enabled' => $zone_enabled[ $i ],
                            );
                        }
                    }

                    if ( (!empty($zone_id[$key]))
                        && ($zone_id[$key] > $this->last_zone_id)
                        && (is_numeric($zone_id[$key]))) {
                        $highest_zone_id = $zone_id[$key];
                        update_option( $this->last_zone_id_option, $highest_zone_id );
                    }

                    update_option( $this->zones_option, $zones );

                    $this->get_zones();

                }

                /* Retrieves zones array from database */
                function get_zones()
                {
                    $this->zones = array_filter( (array) get_option( $this->zones_option ) );
                }

                /* Retrieves last zone id from database */
                function get_last_zone_id()
                {
                    $this->last_zone_id = (int)get_option( $this->last_zone_id_option );
                }

                /* Process and save submitted table_rates */
                function process_table_rates()
                {
                    // Save the rates
                    $table_rate_id = array();
                    $table_rate_zone = array();
                    $table_rate_basis = array();
                    $table_rate_min = array();
                    $table_rate_max = array();
                    $table_rate_cost = array();
                    $table_rate_enabled = array();

                    $table_rates = array();

                    if ( isset( $_POST[ $this->id . '_table_rate_id'] ) ) $table_rate_id = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_id'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_zone'] ) ) $table_rate_zone = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_zone'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_basis'] ) ) $table_rate_basis = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_basis'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_min'] ) )   $table_rate_min   = array_map( 'stripslashes', $_POST[ $this->id . '_table_rate_min'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_max'] ) )   $table_rate_max   = array_map( 'stripslashes', $_POST[ $this->id . '_table_rate_max'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_cost'] ) )  $table_rate_cost  = array_map( 'stripslashes', $_POST[ $this->id . '_table_rate_cost'] );
                    if ( isset( $_POST[ $this->id . '_table_rate_enabled'] ) ) $table_rate_enabled = array_map( 'wc_clean', $_POST[ $this->id . '_table_rate_enabled'] );

                    // Get max key
                    $values = $table_rate_id;
                    ksort( $values );
                    $value = end( $values );
                    $key = key( $values );

                    for ( $i = 0; $i <= $key; $i++ ) {
                        if ( isset( $table_rate_id[ $i ] )
                            && isset( $table_rate_zone[ $i ] )
                            && isset( $table_rate_basis[ $i ] )
                            && isset( $table_rate_min[ $i ] )
                            && isset( $table_rate_max[ $i ] )
                            && isset( $table_rate_cost[ $i ] )
                            && isset( $table_rate_enabled[ $i ] ) ) {

                            $table_rate_cost[ $i ] = wc_format_decimal( $table_rate_cost[$i] );

                            // Add table_rates to array
                            $table_rates[] = array(
                                'id' => $table_rate_id[ $i ],
                                'zone' => $table_rate_zone[ $i ],
                                'basis' => $table_rate_basis[ $i ],
                                'min' => $table_rate_min[ $i ],
                                'max' => $table_rate_max[ $i ],
                                'cost' => $table_rate_cost[ $i ],
                                'enabled' => $table_rate_enabled[ $i ],
                            );
                        }
                    }

                    if ( (!empty($table_rate_id[$key]))
                        && ($table_rate_id[$key] > $this->last_table_rate_id)
                        && (is_numeric($table_rate_id[$key]))) {
                        $highest_table_rate_id = $table_rate_id[$key];
                        update_option( $this->last_table_rate_id_option, $highest_table_rate_id );
                    }

                    update_option( $this->table_rate_option, $table_rates );

                    $this->get_table_rates();

                }

                /* Retrieves table_rates array from database */
                function get_table_rates()
                {
                    $this->table_rates = array_filter( (array) get_option( $this->table_rate_option ) );
                }

                /* Retrieves last table_rate id from database */
                function get_last_table_rate_id()
                {
                    $this->last_table_rate_id = (int)get_option( $this->last_table_rate_id_option );
                }

                /* Retrieves available zone ids for supplied shipping address */
                function get_available_zones($package)
                {

                    $destination_country = $package['destination']['country'];
                    $available_zones = array();

                    foreach ($this->zones as $zone):
                        if ( !empty($zone['country']) && in_array($destination_country, $zone['country']) ) {
                            $available_zones[] = $zone['id'];
                        }
                    endforeach;

                    if (empty($available_zones)) {
                        $found = false;
                        foreach (WC()->countries->get_shipping_countries() as $id => $value):
                            if ($destination_country == $id) {
                                $found = true;
                            }
                        endforeach;
                        if ($found) {
                            $available_zones[] = '0'; // "Everywhere else" zone
                        }
                    }

                    return $available_zones;
                }

                /* Retrieves available table_rates for cart and supplied shipping addresss */
                function get_available_table_rates($package)
                {
                    global $woocommerce;

                    $available_zones = $this->get_available_zones($package);
                    $available_table_rates = array();
                    $weight = $this->get_cart_weight($package);
                    if (WC()->version >= '3.2.0') {
                        $cart_taxes = array_sum($woocommerce->cart->get_cart_contents_taxes());
                    } else {
                        $cart_taxes = array_sum($woocommerce->cart->taxes);
                    }
                    $total = $woocommerce->cart->cart_contents_total + $cart_taxes;

                    foreach ($this->table_rates as $table_rate) {
                        // Is table_rate for an available zone?
                        $zone_pass = (in_array($table_rate['zone'], $available_zones));

                        // Is table_rate valid for basket weight?
                        if ($table_rate['basis'] == 'weight') {
                            $weight_pass = (($weight >= $table_rate['min']) && ($this->is_less_than($weight, $table_rate['max'])));
                        } else {
                            $weight_pass = true;
                        }

                        // Is table_rate valid for basket total?
                        if ($table_rate['basis'] == 'price') {
                            $total_pass = (($total >= $table_rate['min']) && ($this->is_less_than($total, $table_rate['max'])));
                        } else {
                            $total_pass = true;
                        }

                        // Accept table_rate if passes all tests
                        if ($zone_pass && $weight_pass && $total_pass) {
                            $available_table_rates[] = $table_rate;
                        }
                    }
                    return $available_table_rates;
                }

                /* Retrieves total cart weight */
                function get_cart_weight($package) {
                    $total = 0;
                    foreach ( $package['contents'] as $cart_item_key => $cart_item ) {
                        $product = $cart_item['data'];
                        $total += ((float)$product->get_weight() * $cart_item['quantity']);
                    }
                    return $total;
                }

                /* Return true if value less than max, incl. "*" */
                function is_less_than($value, $max)
                {
                    if ($max == '*')
                        return true;
                    else
                        return ($value <= $max);
                }

                /* Retrieves an array item by searching for an id value */
                function find_by_id($array, $id)
                {
                    foreach($array as $a):
                        if ($a['id'] == $id)
                            return $a;
                    endforeach;
                    return false;
                }

                /* Retrieves cheapest rate from a list of table_rates. */
                function pick_cheapest_table_rate($table_rates)
                {
                    $cheapest = false;
                    foreach ($table_rates as $table_rate):
                        if ($cheapest == false)
                            $cheapest = $table_rate;
                        else{
                            if ($table_rate['cost'] < $cheapest['cost'])
                                $cheapest = $table_rate;
                        }
                    endforeach;
                    return $cheapest;
                }

                /* Calculate shipping cost. This is called by WooCommerce */
                public function calculate_shipping($package = array())
                {
                    global $woocommerce;

                    $available_table_rates = $this->get_available_table_rates($package);
                    $table_rate = $this->pick_cheapest_table_rate($available_table_rates);

                    if (!isset($this->settings['tax_status']) || $this->settings['tax_status'] == 'none')
                        $tax = false;
                    else
                        $tax = '';

                    if ($table_rate != false)
                    {
                        /* Handling fee calculation */
                        if (!empty($this->settings['handling_fee']) && $this->settings['handling_fee'] > 0) {
                            $cost = $table_rate['cost'] + $this->settings['handling_fee'];
                        } else {
                            $cost = $table_rate['cost'];
                        }

                        /* Overcost calculation */
                        $cdts = dpdfrance_relais_getOrderShippingCost($package['destination']['postcode'], $cost);

                        /* Coupon override */
                        if ($this->settings['coupon_freeshipping'] == 'enabled') {
                            if ($coupons = WC()->cart->get_coupons()) {
                                foreach ($coupons as $code => $coupon) {
                                    if ($coupon->is_valid() && $coupon->get_free_shipping()) {
                                        $cdts['cost'] = 0;
                                    }
                                }
                            }
                        }

                        /* Franco override */
                        if (!empty($this->settings['franco']) && $this->settings['franco'] > 0) {
                            if (WC()->version >= '3.2.0') {
                                $cart_taxes = array_sum($woocommerce->cart->get_cart_contents_taxes());
                            } else {
                                $cart_taxes = array_sum($woocommerce->cart->taxes);
                            }
                            $total = $woocommerce->cart->cart_contents_total + $cart_taxes;
                            if ($total >= $this->settings['franco']) {
                                $cdts['cost'] = 0;
                            }
                        }

                        /* Max weight override */
                        if (!empty($this->settings['max_weight']) && $this->settings['max_weight'] > 0) {
                            $weight = $this->get_cart_weight($package);
                            if ($weight >= $this->settings['max_weight']) {
                                $cdts['excl'] = 2;
                            }
                        }

                        /* Exclude shipping classes */
                        if (!empty($this->settings['classes_to_exclude'])) {
                            $cart_shipping_classes = dpdfrance_relais_find_shipping_classes($package);
                            $classes_to_exclude = explode(',', $this->settings['classes_to_exclude']);
                            if (array_intersect($classes_to_exclude, $cart_shipping_classes)) {
                                $cdts['excl'] = 3;
                            }
                        }

                        /* Register the rate */
                        $rate = array(
                            'id' => $this->id,
                            'label' => $this->title.($cdts['cost'] == 0 ? ': '.__('free', 'woocommerce-dpdfrance') : ''),
                            'cost' => $cdts['cost'],
                            'taxes' => $tax,
                            'calc_tax' => 'per_order'
                        );
                        if ($cdts['excl'] == 0)
                            $this->add_rate($rate);
                    }
                }
            }
        }
        /* End of class */

        /* Add shipping mehod */
        function add_dpdfrance_relais_delivery_method($methods) {
            $methods[] = 'WC_dpdfrance_relais';
            return $methods;
        }

        /* Hide shipping cost 0.00 when free shipping */
        add_filter( 'woocommerce_cart_shipping_method_full_label', function( $label, $method ) {
            if ($method->id == 'dpdfrance_relais' && $method->cost <= 0 ) {
                $label = $method->get_label();
            }
            return $label;}, 10, 2 );
        add_filter('woocommerce_shipping_methods', 'add_dpdfrance_relais_delivery_method');
    }

    /* Load Translation files */
    function woocommerce_dpdfrance_relais_load_plugin_textdomain()
    {
        load_plugin_textdomain( 'woocommerce-dpdfrance', FALSE, basename(dirname(__FILE__)) . '/../../../languages/plugins/');
    }
    add_action( 'init', 'woocommerce_dpdfrance_relais_load_plugin_textdomain' );

    /* Loads custom JS file */
    function woocommerce_dpdfrance_relais_custom_script()
    {
        wp_enqueue_script( 'dpdfrance_relais_custom_script', OS_DPDFRANCE_ROOT_URL.'/assets/js/front/relais/dpdfrance_relais.js', array('jquery'), '0.3' );
        wp_enqueue_script( 'dpdfrance_relais_custom_map', 'https://maps.googleapis.com/maps/api/js?key='.get_option('wc_settings_tab_dpdfrance_google_api_key'), '');
        wp_enqueue_style( 'dpdfrance_relais_custom_css', OS_DPDFRANCE_ROOT_URL.'/assets/css/front/dpdfrance_shipping.css');
    }
    add_action('woocommerce_checkout_shipping', 'woocommerce_dpdfrance_relais_custom_script');

    /* [FO] Checks if selected carrier is dpdfrance_relais, and controls value of shipping_company field */
    function woocommerce_dpdfrance_relais_checkout_controller()
    {
        global $woocommerce;
        $html = dpdfrance_relais_front_controller($woocommerce->customer);

        @parse_str($_POST['post_data'], $post_data);

        $postcode = @($post_data['ship_to_different_address'] == 1 ? $post_data['shipping_postcode'] : $post_data['billing_postcode']);
        $cdts = dpdfrance_relais_getOrderShippingCost($postcode);

        if ($woocommerce->session->chosen_shipping_methods[0] == 'dpdfrance_relais') {
            print_r('<tr><td colspan="2">'.$html.'</div></td></tr>');
        } else {
            echo '<script type="text/javascript">
                    if (typeof dpdResetRelaisValues === "function")
                        dpdResetRelaisValues();
                </script>';
        }

    }
    add_action('woocommerce_review_order_after_shipping', 'woocommerce_dpdfrance_relais_checkout_controller');

    function dpdfrance_relais_front_controller($customer_data)
    {
        global $woocommerce, $wpdb;

        /* Init display */
        $html  = '';
        
        // Read cookie and search pudos
        if (isset($_COOKIE['dpdfrance_relais_search']))
            $cookiedata = explode(',', $_COOKIE['dpdfrance_relais_search']);

        $address_search = new StdClass();
        if (!empty($cookiedata[1])) {
            $address_search->address = $cookiedata[0];
            $address_search->postcode = $cookiedata[1];
            $address_search->city = $cookiedata[2];
        } else {
            $address_search->address = (is_wc3() ? $customer_data->get_billing_address_1() : $customer_data->address_1);
            $address_search->postcode = (is_wc3() ? $customer_data->get_billing_postcode() : $customer_data->postcode);
            $address_search->city = (is_wc3() ? $customer_data->get_billing_city() : $customer_data->city);
        }
        $dpd_relais_points = dpdfrance_getDPDPickupPoints($address_search);
        if (!empty($dpd_relais_points['error'])) {
            $html .= '<div class="dpdfrance_relais_error">'.$dpd_relais_points['error'].'</div>';
        } else {
            $html .= '<script type="text/javascript">
                        if (typeof relais_data != "undefined")
                            $("#"+relais_data.relay_id).click();
                        </script>';
            $html .= '<div id="dpdfrance_relais_filter" onclick="
                            $(\'#dpdfrance_relais_filter\').hide();
                            $(\'.dpdfrance_relaisbox\').hide();"></div>
                        <table align="center" id="dpdfrance_relais_point_table" class="dpdfrance_relaistable">';

            $html .= '<tr>
                            <td colspan="2" style="padding:0px;" class="dpdfrance_headertd">
                                <div id="dpdfrance_div_relais_header"><p>'.__('Please select your DPD Pickup point among this list', 'woocommerce-dpdfrance').'</p></div>
                                <div id="dpdfrance_div_relais_srch_link"><span onMouseOver="javascript:this.style.cursor=\'pointer\';javascript:this.style.textDecoration=\'underline\';" onMouseOut="javascript:this.style.cursor=\'auto\';javascript:this.style.textDecoration=\'none\';" onClick="$(\'#dpdfrance_div_relais_srch_panel\').slideToggle();">'.__('Search for Pickup points near another address', 'woocommerce-dpdfrance').'</span>
                                <div id="dpdfrance_div_relais_srch_panel" style="display:none;">
                                    <input type="text" id="dpdfrance_search_address" placeholder="'.__('Address', 'woocommerce-dpdfrance').'" autocomplete="nope"/><br/>
                                    <input type="text" id="dpdfrance_search_zipcode" maxlength="5" placeholder="'.__('Postcode', 'woocommerce-dpdfrance').'" autocomplete="nope"/> <input type="text" id="dpdfrance_search_city" placeholder="'.__('City', 'woocommerce-dpdfrance').'" autocomplete="nope"/>
                                    <div id="dpdfrance_search_buttons">
                                        <button type="button" id="dpdfrance_search_submit" name="dpdfrance_search_submit" onclick="dpdfrance_relais_ajaxupdate($(\'#dpdfrance_search_address\').val(), $(\'#dpdfrance_search_zipcode\').val(), $(\'#dpdfrance_search_city\').val(), \'search\');">'.__('Search', 'woocommerce-dpdfrance').'</button>
                                        <button type="button" id="dpdfrance_reset_submit" name="dpdfrance_reset_submit" onclick="dpdfrance_relais_ajaxupdate($(\'#billing_address_1\').val(), $(\'#billing_postcode\').val(), $(\'#billing_city\').val(), \'reset\');">'.__('Reset', 'woocommerce-dpdfrance').'</button>
                                    </div>
                                </div></div>
                            </td>
                        </tr>';
            if (!empty($dpd_relais_points['empty'])) {
                $html .= '<tr><td colspan="2" style="padding:0px;" class="dpdfrance_headertd"><div class="dpdfrance_relais_error">'.$dpd_relais_points['empty'].'</div></td></tr>';
            } else {
                foreach ($dpd_relais_points as $key => $item)
                {
                    $html .= '
                <tr><td colspan="2" class="dpdfrance_tdpr">
                    <div class="dpdfrance_lignepr">
                        <div align="left" class="dpdfrance_logorelais"></div>
                        <div align="left" class="dpdfrance_adressepr"><b>'.$item['shop_name'].'</b><br/>'.$item['address1'].'<br/>'.$item['postal_code'].' '.$item['city'].'<br/></div>
                        <div align="right" class="dpdfrance_distancepr">'.$item['distance'].' km</div>
                        <div align="center" class="dpdfrance_popinpr">
                            <a href="#!" onClick="openDpdfranceDialog(\''.OS_DPDFRANCE_ROOT_URL.'\',\'dpdfrance_relaydetail'.$key.'\',\'map_canvas'.$key.'\',\''.$item['coord_lat'].'\',\''.$item['coord_long'].'\')">'.__('More details', 'woocommerce-dpdfrance').'</a>
                        </div>
                        <div align="center" class="dpdfrance_radiopr">
                            <input onclick="dpdWriteRelaisValues(document.getElementById(\''.$item['relay_id'].'\'))" type="radio" name="dpdfrance_relay_id" id="'.$item['relay_id'].'" value=\''.json_encode($item).'\'></input>
                            <label for="'.$item['relay_id'].'"><span><span></span></span><b>'.__('ICI', 'woocommerce-dpdfrance').'</b></label>
                        </div>
                    </div>
                </td></tr>';

                    $html .= '<div class="dpdfrance_relaisbox" id="dpdfrance_relaydetail'.$key.'" style="display:none;">
                        <div class="dpdfrance_relaisboxclose" onclick="
                            document.getElementById(\'dpdfrance_relaydetail'.$key.'\').style.display=\'none\';
                            document.getElementById(\'dpdfrance_relais_filter\').style.display=\'none\'">
                            <img src="'.OS_DPDFRANCE_ROOT_URL.'/assets/img/front/relais/box-close.png"/>
                        </div>
                                <div class="dpdfrrelaisboxcarto" id="map_canvas'.$key.'" style="width:100%;"></div>
                                <div id="dpdfrrelaisboxbottom" class="dpdfrrelaisboxbottom">
                                <div id="dpdfrrelaisboxadresse" class="dpdfrrelaisboxadresse">
                                    <div class="dpdfrrelaisboxadresseheader"><img src="'.OS_DPDFRANCE_ROOT_URL.'/assets/img/front/relais/pointrelais.png" alt="-" width="32" height="32"/><br/>'.__('Your DPD Pickup point', 'woocommerce-dpdfrance').'</div>
                                    <strong>'.$item['shop_name'].'</strong></br>
                                    '.$item['address1'].'</br>';
                                    if (!empty($item['address2']))
                                        $html .= $item['address2'].'</br>';
                                    $html .= $item['postal_code'].'  '.$item['city'].'<br/>';
                                    if (!empty($item['local_hint']))
                                        $html .= '<p>'.__('Landmark', 'woocommerce-dpdfrance').' : '.$item['local_hint'].'</p>';
                         $html .= '</div>';

                    $html .= '<div class="dpdfrrelaisboxhoraires">
                                <div class="dpdfrrelaisboxhorairesheader"><img src="'.OS_DPDFRANCE_ROOT_URL.'/assets/img/front/relais/horaires.png" alt="-" width="32" height="32"/><br/>'.__('Opening Hours', 'woocommerce-dpdfrance').'</div>
                                <p><span>'.__('Monday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['monday'].'</p>
                                <p><span>'.__('Tuesday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['tuesday'].'</p>
                                <p><span>'.__('Wednesday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['wednesday'].'</p>
                                <p><span>'.__('Thursday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['thursday'].'</p>
                                <p><span>'.__('Friday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['friday'].'</p>
                                <p><span>'.__('Saturday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['saturday'].'</p>
                                <p><span>'.__('Sunday', 'woocommerce-dpdfrance').' : </span>'.$item['opening_hours']['sunday'].'</p>
                            </div>';

                    $html .= '<div class="dpdfrrelaisboxinfos">
                                <div class="dpdfrrelaisboxinfosheader"><img src="'.OS_DPDFRANCE_ROOT_URL.'/assets/img/front/relais/info.png" alt="-" width="32" height="32"/><br/>'.__('More info', 'woocommerce-dpdfrance').'</div>
                                <div><h5>'.__('Distance in KM', 'woocommerce-dpdfrance').' : </h5><strong>'.$item['distance'].' km </strong></div>
                                <div><h5>'.__('DPD Pickup ID#', 'woocommerce-dpdfrance').' : </h5><strong>'.(string)$item['relay_id'].'</strong></div>';
                            if (!empty($item['closing_period']) && count($item['closing_period']) > 0)
                            {
                                foreach ($item['closing_period'] as $holiday_item)
                                {
                                    $holiday_item = (array)$holiday_item;
                                    $html .= '<div><img id="dpdfrrelaisboxinfoswarning" src="'.OS_DPDFRANCE_ROOT_URL.'/assets/img/front/relais/warning.png" alt="-" width="16" height="16"/> <h4>'.__('Closing period', 'woocommerce-dpdfrance').' : </h4> '.$holiday_item[0].'</div>';
                                }
                            }
                        $html .= '</div>';

                    $html .= '</div></div>'; // dpdfrrelaisboxbottom et relaydetail
                }
            }
            $html .= '</table>';
            $html .= '<script type="text/javascript">
                        if (typeof $ === "function" && $("input[name=dpdfrance_relay_id]:checked").length == 0)
                            $("input[name=dpdfrance_relay_id]").first().click();
                    </script>';
        }
        /* Display */
        return $html;
    }

    /**
     * Affiche la page
     *
     * @access public
     * @param mixed $atts
     * @return void
     */
    function os_iframe_dpdfrance( $atts )
    {
        global $woocommerce;

        $woocommerce->nocache();
        $woocommerce->show_messages();
    }
    
    /* Return true if WooCommerce version is 3 and up */
    function is_wc3()
    {
        if (WC()->version >= '3.0.0') {
            return true;
        } else {
            return false;
        }
    }
    
    /* Retrieves Pickup points */
    function dpdfrance_getDPDPickupPoints($input)
    {
        $my_WC_dpdfrance_relais = new WC_dpdfrance_relais();

        $dpd_relais_points = array();

        $serviceurl     = $my_WC_dpdfrance_relais->dpdfrance_relais_MyPudoURL;
        $carrier        = $my_WC_dpdfrance_relais->dpdfrance_relais_MerchantID;
        $key            = $my_WC_dpdfrance_relais->dpdfrance_relais_SecurityKey;
        $date           = date('d/m/Y');
        $address        = dpdfrance_relais_stripAccents($input->address);
        $zipcode        = $input->postcode;
        $city           = dpdfrance_relais_stripAccents($input->city);
        // Zip code is mandatory
        if (empty($zipcode))
        {
            $dpd_relais_points['error'] = __('Postal code in missing in the address. Please, modify it.', 'woocommerce-dpdfrance');
            return $dpd_relais_points;
        }

        // MyPudo call parameters
        $variables = array(
            'carrier'       => $carrier,
            'key'           => $key,
            'address'       => $address,
            'zipCode'       => $zipcode,
            'city'          => $city,
            'countrycode'   =>'FR',
            'requestID'     =>'1234',
            'request_id'    =>'1234',
            'date_from'     =>$date,
            'max_pudo_number'=>'',
            'max_distance_search'=>'',
            'weight'        =>'',
            'category'      =>'',
            'holiday_tolerant'=>''
        );

        try{
            ini_set('default_socket_timeout', 5);
            $soappudo = new SoapClient($serviceurl, array('connection_timeout' => 5, 'cache_wsdl' => WSDL_CACHE_NONE, 'exceptions' => true));
            $GetPudoList = $soappudo->getPudoList($variables)->GetPudoListResult->any;

            // Get the webservice XML response and parse its values
            $xml = new SimpleXMLElement($GetPudoList);
                if (strlen($xml->ERROR) > 0)
                    $dpd_relais_points['error'] = __('DPD Relais is not available at the moment, please try again shortly.', 'woocommerce-dpdfrance');
                else
                {
                    $relais_items = $xml->PUDO_ITEMS;
                    // Loop through each pudo
                    $i = 0;
                    foreach ($relais_items->PUDO_ITEM as $item)
                    {
                        $point = array();
                        $item = (array)$item;
                        $cdts = dpdfrance_relais_getOrderShippingCost($item['ZIPCODE']);
                        if ($cdts['excl'] != 0)
                            continue;
                        $point['relay_id']       = $item['PUDO_ID'];
                        $point['shop_name']      = dpdfrance_relais_stripAccents($item['NAME']);
                        $point['address1']       = dpdfrance_relais_stripAccents($item['ADDRESS1']);
                        if ($item['ADDRESS2'] != '')
                            $point['address2']   = dpdfrance_relais_stripAccents($item['ADDRESS2']);
                        if ($item['ADDRESS3'] != '')
                            $point['address3']   = dpdfrance_relais_stripAccents($item['ADDRESS3']);
                        if ($item['LOCAL_HINT'] != '')
                            $point['local_hint'] = dpdfrance_relais_stripAccents($item['LOCAL_HINT']);
                        $point['postal_code']    = $item['ZIPCODE'];
                        $point['city']           = dpdfrance_relais_stripAccents($item['CITY']);
                        $point['distance']       = number_format($item['DISTANCE'] / 1000, 2);
                        $point['coord_lat']      = (float)strtr($item['LATITUDE'], ',', '.');
                        $point['coord_long']     = (float)strtr($item['LONGITUDE'], ',', '.');

                        $days=array(1=>'monday',2=>'tuesday',3=>'wednesday',4=>'thursday',5=>'friday',6=>'saturday',7=>'sunday');
                        if(count($item['OPENING_HOURS_ITEMS']->OPENING_HOURS_ITEM)>0)
                            foreach($item['OPENING_HOURS_ITEMS']->OPENING_HOURS_ITEM as $k=>$oh_item)
                            {
                                $oh_item=(array)$oh_item;
                                $point[$days[$oh_item['DAY_ID']]][]=$oh_item['START_TM'].' - '.$oh_item['END_TM'];
                            }

                        if(empty($point['monday'])){$h1 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['monday'][1])){$h1 = $point['monday'][0];}
                                else{$h1 = $point['monday'][0].' & '.$point['monday'][1];}}

                        if(empty($point['tuesday'])){$h2 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['tuesday'][1])){$h2 = $point['tuesday'][0];}
                                else{$h2 = $point['tuesday'][0].' & '.$point['tuesday'][1];}}

                        if(empty($point['wednesday'])){$h3 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['wednesday'][1])){$h3 = $point['wednesday'][0];}
                                else{$h3 = $point['wednesday'][0].' & '.$point['wednesday'][1];}}

                        if(empty($point['thursday'])){$h4 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['thursday'][1])){$h4 = $point['thursday'][0];}
                                else{$h4 = $point['thursday'][0].' & '.$point['thursday'][1];}}

                        if(empty($point['friday'])){$h5 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['friday'][1])){$h5 = $point['friday'][0];}
                                else{$h5 = $point['friday'][0].' & '.$point['friday'][1];}}

                        if(empty($point['saturday'])){$h6 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['saturday'][1])){$h6 = $point['saturday'][0];}
                                else{$h6 = $point['saturday'][0].' & '.$point['saturday'][1];}}

                        if(empty($point['sunday'])){$h7 = __('Closed', 'woocommerce-dpdfrance');}
                            else{if(empty($point['sunday'][1])){$h7 = $point['sunday'][0];}
                                else{$h7 = $point['sunday'][0].' & '.$point['sunday'][1];}}

                        $point['opening_hours'] = array('monday' => $h1, 'tuesday' => $h2, 'wednesday' => $h3, 'thursday' => $h4, 'friday' => $h5, 'saturday' => $h6, 'sunday' => $h7);

                        if (count($item['HOLIDAY_ITEMS']->HOLIDAY_ITEM) > 0)
                            $x = 0;
                            foreach ($item['HOLIDAY_ITEMS']->HOLIDAY_ITEM as $holiday_item)
                            {
                                $holiday_item = (array)$holiday_item;
                                $point['closing_period'][$x] = $holiday_item['START_DTM'].' - '.$holiday_item['END_DTM'];
                                ++$x;
                            }
                        array_push($dpd_relais_points, $point);
                        if (++$i == 5)
                            break;
                    }
                }
        } catch (Exception $e){
            $dpd_relais_points['error'] = __('DPD Relais is not available at the moment, please try again shortly.', 'woocommerce-dpdfrance');
        }
        if (empty($dpd_relais_points))
            $dpd_relais_points['empty'] = __('There are no Pickup points near this address, please modify it.', 'woocommerce-dpdfrance');
        return $dpd_relais_points;
    }

    /* Replaces accented characters and symbols */
    function dpdfrance_relais_stripAccents($str)
    {
        $str = preg_replace('/[\x{00C0}\x{00C1}\x{00C2}\x{00C3}\x{00C4}\x{00C5}]/u', 'A', $str);
        $str = preg_replace('/[\x{0105}\x{0104}\x{00E0}\x{00E1}\x{00E2}\x{00E3}\x{00E4}\x{00E5}]/u', 'a', $str);
        $str = preg_replace('/[\x{00C7}\x{0106}\x{0108}\x{010A}\x{010C}]/u', 'C', $str);
        $str = preg_replace('/[\x{00E7}\x{0107}\x{0109}\x{010B}\x{010D}}]/u', 'c', $str);
        $str = preg_replace('/[\x{010E}\x{0110}]/u', 'D', $str);
        $str = preg_replace('/[\x{010F}\x{0111}]/u', 'd', $str);
        $str = preg_replace('/[\x{00C8}\x{00C9}\x{00CA}\x{00CB}\x{0112}\x{0114}\x{0116}\x{0118}\x{011A}]/u', 'E', $str);
        $str = preg_replace('/[\x{00E8}\x{00E9}\x{00EA}\x{00EB}\x{0113}\x{0115}\x{0117}\x{0119}\x{011B}]/u', 'e', $str);
        $str = preg_replace('/[\x{00CC}\x{00CD}\x{00CE}\x{00CF}\x{0128}\x{012A}\x{012C}\x{012E}\x{0130}]/u', 'I', $str);
        $str = preg_replace('/[\x{00EC}\x{00ED}\x{00EE}\x{00EF}\x{0129}\x{012B}\x{012D}\x{012F}\x{0131}]/u', 'i', $str);
        $str = preg_replace('/[\x{0142}\x{0141}\x{013E}\x{013A}]/u', 'l', $str);
        $str = preg_replace('/[\x{00F1}\x{0148}]/u', 'n', $str);
        $str = preg_replace('/[\x{00D2}\x{00D3}\x{00D4}\x{00D5}\x{00D6}\x{00D8}]/u', 'O', $str);
        $str = preg_replace('/[\x{00F2}\x{00F3}\x{00F4}\x{00F5}\x{00F6}\x{00F8}]/u', 'o', $str);
        $str = preg_replace('/[\x{0159}\x{0155}]/u', 'r', $str);
        $str = preg_replace('/[\x{015B}\x{015A}\x{0161}]/u', 's', $str);
        $str = preg_replace('/[\x{00DF}]/u', 'ss', $str);
        $str = preg_replace('/[\x{0165}]/u', 't', $str);
        $str = preg_replace('/[\x{00D9}\x{00DA}\x{00DB}\x{00DC}\x{016E}\x{0170}\x{0172}]/u', 'U', $str);
        $str = preg_replace('/[\x{00F9}\x{00FA}\x{00FB}\x{00FC}\x{016F}\x{0171}\x{0173}]/u', 'u', $str);
        $str = preg_replace('/[\x{00FD}\x{00FF}]/u', 'y', $str);
        $str = preg_replace('/[\x{017C}\x{017A}\x{017B}\x{0179}\x{017E}]/u', 'z', $str);
        $str = preg_replace('/[\x{00C6}]/u', 'AE', $str);
        $str = preg_replace('/[\x{00E6}]/u', 'ae', $str);
        $str = preg_replace('/[\x{0152}]/u', 'OE', $str);
        $str = preg_replace('/[\x{0153}]/u', 'oe', $str);
        $str = preg_replace('/[^\p{L}\p{N}]/u', ' ', $str);
        $str = strtoupper($str);
        return $str;
    }

    /* Island and Mountain zones overcost calculation functions */
    function dpdfrance_relais_getOrderShippingCost($postcode, $cost = 0)
    {
        global $woocommerce;

        $excl = 0;
        $cfg = new WC_dpdfrance_relais();
        $supp_iles = $cfg->settings['supp_iles'];
        $supp_montagne = $cfg->settings['supp_montagne'];

        $zone_iles = array('17111','17123','17190','17310','17370','17410','17480','17550','17580','17590','17630','17650','17670','17740','17840','17880','17940','22870','29242','29253','29259','29980','29990','56360','56590','56780','56840','85350');
        $zone_montagne = array('04120','04130','04140','04160','04170','04200','04240','04260','04300','04310','04330','04360','04370','04400','04510','04530','04600','04700','04850','05100','05110','05120','05130','05150','05160','05170','05200','05220','05240','05250','05260','05290','05300','05310','05320','05330','05340','05350','05400','05460','05470','05500','05560','05600','05700','05800','06140','06380','06390','06410','06420','06430','06450','06470','06530','06540','06620','06710','06750','06910','09110','09140','09300','09460','25120','25140','25240','25370','25450','25500','25650','30570','31110','38112','38114','38142','38190','38250','38350','38380','38410','38580','38660','38700','38750','38860','38880','39220','39310','39400','63113','63210','63240','63610','63660','63690','63840','63850','64440','64490','64560','64570','65110','65120','65170','65200','65240','65400','65510','65710','66210','66760','66800','68140','68610','68650','73110','73120','73130','73140','73150','73160','73170','73190','73210','73220','73230','73250','73260','73270','73300','73320','73340','73350','73390','73400','73440','73450','73460','73470','73500','73530','73550','73590','73600','73620','73630','73640','73710','73720','73870','74110','74120','74170','74220','74230','74260','74310','74340','74350','74360','74390','74400','74420','74430','74440','74450','74470','74480','74660','74740','74920','83111','83440','83530','83560','83630','83690','83830','83840','84390','88310','88340','88370','88400','90200');

        if (substr($postcode, 0, 2) == '20' || in_array($postcode, $zone_iles)) {
            if ($supp_iles >= 0)
                $cost = $cost + $cfg->settings['supp_iles'];
            else
                $excl = 1;
        }

        if (in_array($postcode, $zone_montagne)) {
            if ($supp_montagne >= 0)
                $cost = $cost + $cfg->settings['supp_montagne'];
            else
                $excl = 2;
        }

        return array('excl' => $excl, 'cost' => $cost);
    }

    /* Get the shipping classes inside cart*/
    function dpdfrance_relais_find_shipping_classes($package) {
        $found_shipping_classes = array();
        foreach ( $package['contents'] as $item_id => $values ) {
            if ( $values['data']->needs_shipping() ) {
                $found_shipping_classes[$item_id] = $values['data']->get_shipping_class();
            }
        }
        return $found_shipping_classes;
    }
}